#!/usr/bin/env python3
# -*- coding: utf-8 -*-
from .yaml_util import read_yaml

from qiniu import put_file, Auth, BucketManager, build_batch_delete


class QiniuFunction(object):
    def __init__(self, access_key=read_yaml('access_key', 'config'), secret_key=read_yaml('secret_key', 'config'),
                 bucket=str(read_yaml('bucket_name', 'config'))):
        self.bucket = bucket
        self.q = Auth(access_key, secret_key)
        self.domain = read_yaml('domain', 'config')

    def upload_file(self, local_file, remote_file):
        # 上传后保存的文件名
        token = self.q.upload_token(self.bucket, remote_file, 999999)
        ret, info = put_file(token, remote_file, local_file, version='v2')
        return ret, info

    def download_file(self, remote_file):
        # 有两种方式构造base_url的形式
        base_url = 'https://%s/%s' % (self.domain, remote_file)
        # private_url = self.q.private_download_url(base_url, expires=3600)
        return base_url

    def delete_file(self, remote_file):
        bucket_manager = BucketManager(self.q)
        ret, info = bucket_manager.delete(self.bucket, remote_file)
        return ret, info

    def delete_files(self, remote_files):
        bucket_manager = BucketManager(self.q)
        ops = build_batch_delete(self.bucket, remote_files)
        ret, info = bucket_manager.batch(ops)
        return ret, info



# -*- coding: utf-8 -*-            
# @Author : buyfakett
# @Time : 2023/12/28 11:26
import dns.resolver


def check_domain(domain):
    """
    检测域名是否做了dns解析
    :param domain:      域名
    :return:            bool
    """
    try:
        dns.resolver.query(domain, 'A')
        return True
    except dns.resolver.NXDOMAIN:
        return False


# -*- coding: utf-8 -*-            
# @Author : buyfakett
# @Time : 2023/11/9 9:37

import base64
from Crypto.Cipher import AES


def addStrToSpecifyLen(s, specifyLen=0):
    """
    s不是specifyLen的倍数那就补足为specifyLen的倍数
    :param s: 需要加密的参数
    :param specifyLen: 指定参数的位数
    :return: 补足位数的参数
    """
    if specifyLen <= 0:
        specifyLen = 1;
    while len(s) % specifyLen != 0:
        s += '\0'
    return s.encode(encoding='utf-8')


def encrypt_aes(data: str = '', aes_key: str = ''):
    """
    aes的ecb模式加密
    :param data: 加密数据
    :param aes_key: 加密的秘钥
    :return: 加密之后的密文
    """
    # 初始化加密器
    aes = AES.new(addStrToSpecifyLen(aes_key, 16), AES.MODE_ECB)
    # 先进行aes加密
    encrypt = aes.encrypt(addStrToSpecifyLen(data, 16))
    # 用base64转成字符串形式
    encrypted_text = str(base64.encodebytes(encrypt), encoding='utf-8').replace('\n', '')  # 执行加密并转码返回bytes
    return encrypted_text


def decrypt_aes(data: str, aes_key: str = ''):
    """
    aes的ecb模式解密
    :param data: 待解密数据
    :param aes_key: 加密的秘钥
    :return: 解密之后的数据
    """
    # 初始化加密器
    aes = AES.new(addStrToSpecifyLen(aes_key, 16), AES.MODE_ECB)
    # 优先逆向解密base64成bytes
    base64_decrypted = base64.decodebytes(addStrToSpecifyLen(data, 16))
    # 执行解密密并转码返回str
    decrypted_text = str(aes.decrypt(base64_decrypted), encoding='utf-8').replace('\0', '')
    return decrypted_text



# -*- coding: utf-8 -*-            
# @Author : buyfakett
# @Time : 2023/11/21 17:21
import fnmatch
import logging
import os
import subprocess
import time


def exec_shell(command: str):
    """
    :param command:     string      命令
    :return output: string      日志
    :return error: string      错误
    :return returncode: string      返回码
    """
    process = subprocess.Popen(command, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    output, error = process.communicate()
    return output.decode('utf-8'), error.decode('utf-8'), process.returncode


def check_file(directory: str, filename: str, max_checks: int = 60):
    """
    检测服务器是否有这个文件
    :param directory:           检查的目录
    :param filename:            检查的文件名
    :param max_checks:          最多检测的次数
    """
    checks = 1
    logging.info(f'本次检查{directory}下的{filename}文件，一共检测{max_checks}次')

    while checks < max_checks:
        # 检测文件夹本身是否存在
        if not os.path.exists(directory):
            logging.info(f"当前为第{checks}次尝试，文件夹 {directory} 不存在，等待...")
            time.sleep(5)
            checks += 1
        else:
            # 检测文件是否存在
            files = os.listdir(directory)
            matched_files = [file for file in files if fnmatch.fnmatch(file, filename)]

            if matched_files:
                logging.info(f"当前为第{checks}次尝试，文件 {filename} 存在，继续执行！")
                return True  # 如果文件存在，返回成功
            else:
                logging.info(f"当前为第{checks}次尝试，文件 {filename} 不存在，等待...")
                time.sleep(5)
                checks += 1
    return False  # 达到最大检测次数后返回失败



import os, yaml, logging


# 读取yaml
def read_yaml(key: str, filename: str):
    """
    读取yaml配置文件
    :param key:     string      要读取的值的名称
    :param filename: string     配置文件的名字
    :return         *           要读取的值
    """
    try:
        with open(os.getcwd() + '/config/' + filename + '.yaml', encoding="utf-8") as f:
            value = yaml.load(stream=f, Loader=yaml.FullLoader)
            return value[key]
    except:
        logging.error(f"{filename}.yaml文件中没有{key}配置，请检查配置文件！！！")
        raise SystemExit(0)


# 追加写入yaml
def write_yaml(data: str, filename: str):
    """
    写入yaml配置
    :param data:        json格式
    :param filename:    配置文件的名字
    """
    with open(os.getcwd() + '/config/' + filename, encoding="utf-8", mode="a") as f:
        yaml.dump(data=data, stream=f, allow_unicode=True)


# 写入yaml
def write_yaml_value(data: str, filename: str):
    """
    写入yaml配置
    :param data:        json格式
    :param filename:    配置文件的名字
    """
    with open(os.getcwd() + '/config/' + filename, encoding="utf-8", mode="w") as f:
        yaml.safe_dump(data=data, stream=f, allow_unicode=True)


# 清空yaml
def clear_yaml(filename: str):
    """
    清空yaml
    :param filename:    配置文件的名字
    """
    with open(os.getcwd() + '/config/' + filename, encoding="utf-8", mode="w") as f:
        f.truncate()


# 读取全部yaml配置
def read_all_yaml(filename: str):
    """
    读取全部yaml配置
    :param filename:    配置文件的名字
    :return:            全部配置
    """
    with open(os.getcwd() + '/config/' + filename, encoding="utf-8") as f:
        value: object = yaml.load(stream=f, Loader=yaml.FullLoader)
        return value



import logging
import subprocess


class SVNClient:
    """
    :param repo_url:     string      svn地址
    :param working_copy_path: string     检出地址
    :param username: string     用户名
    :param password: string     密码
    :return output: string      日志
    :return error: string      错误
    :return returncode: string      返回码
    """

    def __init__(self, repo_url: str, working_copy_path: str, username: str, password: str):
        self.repo_url = repo_url
        self.working_copy_path = working_copy_path
        self.username = username
        self.password = password

    def execute_svn_command(self, command):
        full_command = f'svn {command}'
        process = subprocess.Popen(full_command, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        output, error = process.communicate()
        return output.decode('utf-8'), error.decode('utf-8'), process.returncode

    def checkout(self):
        return self.execute_svn_command(
            f'checkout {self.repo_url} {self.working_copy_path} --username={self.username} --password={self.password}')

    def update(self):
        return self.execute_svn_command(
            f'update {self.working_copy_path} --username={self.username} --password={self.password}')

    def add(self, route: str):
        return self.execute_svn_command(
            f'add {route}')

    def commit(self, message: str):
        return self.execute_svn_command(
            f'commit -m "{message}" {self.working_copy_path} --username={self.username} --password={self.password}')



import os
import io
from setuptools import setup, find_packages

with io.open("README.md", 'r', encoding='utf-8') as f:
    long_description = f.read()

with io.open("requirements.txt", 'r') as f:
    install_requires = f.read().split(os.sep)


setup(
    name='tt_util',
    version='1.1.5',
    description='buyfakett自用的工具包',
    author='buyfakett',
    author_email='buyfakett@vip.qq.com',
    license='MIT',
    url="https://github.com/buyfakett",
    packages=find_packages(),  # packages=["pytest"],
    long_description=long_description,
    long_description_content_type='text/markdown',
    include_package_data=True,
    zip_safe=True,
    python_requires='>=3.6',
    install_requires=install_requires,
)



import logging
import paramiko
import time


class SSHClient:
    """
    :param host:     string      ip
    :param password: string     密码
    :param port: string     端口
    :param username: string     用户名
    """

    def __init__(self, host: str,
                 password: str = None, port: int = 22, username: str = 'root'):

        self.host = host
        self.port = port
        self.username = username
        self.password = password
        self.client = paramiko.SSHClient()
        self.client.set_missing_host_key_policy(paramiko.AutoAddPolicy())
        self.connect()

    def connect(self):
        self.client.connect(self.host, port=self.port, username=self.username, password=self.password)

    def execute_command(self, command):
        logging.info(f"即将执行命令: {command}")
        stdin, stdout, stderr = self.client.exec_command(command)
        output = stdout.read().decode('utf-8')
        error = stderr.read().decode('utf-8')
        exit_code = stdout.channel.recv_exit_status()
        logging.info(f"执行结果:\n{output}")
        if error:
            logging.error(f"执行错误:\n{error}")
        logging.info(f"退出代码: {exit_code}")
        return {
            'stdout': output,
            'stderr': error,
            'exit_code': exit_code
        }

    def execute_commands(self, commands):
        results = []
        for command in commands:
            result = self.execute_command(command)
            results.append(result)
        return results

    def upload_and_execute_script(self, local_path, remote_path):
        logging.info("----------------开始脚本文件----------------")
        sftp = self.client.open_sftp()
        sftp.put(local_path, remote_path)
        sftp.close()
        logging.info("----------------上传脚本结束----------------")

        # 确保脚本是可执行的
        self.execute_command(f'chmod +x {remote_path}')
        self.execute_command(f"sed -i 's/\r//' {remote_path}")

        # 执行脚本
        command_result = self.execute_command(f'/bin/bash {remote_path}')
        return command_result

    def upload_file(self, local_path, remote_path):
        logging.info("----------------开始上传文件----------------")
        sftp = self.client.open_sftp()
        sftp.put(local_path, remote_path)
        sftp.close()
        logging.info("----------------上传文件结束----------------")

    def download_file(self, remote_path, local_path):
        logging.info("----------------开始下载文件----------------")
        sftp = self.client.open_sftp()
        sftp.get(remote_path, local_path)
        sftp.close()
        logging.info("----------------下载文件结束----------------")

    def wait_for_file(self, remote_path, timeout=300):
        start_time = time.time()
        i = 0

        while time.time() - start_time < timeout:
            sftp = self.client.open_sftp()
            i = i + 1

            try:
                sftp.stat(remote_path)
                logging.info(f"当前为第{i}次尝试，文件 {remote_path} 存在，可以执行下一步操作。")
                sftp.close()
                return True
            except FileNotFoundError:
                logging.info(f"当前为第{i}次尝试，文件 {remote_path} 不存在，等待...")
                sftp.close()
                time.sleep(5)  # 休眠5秒，可以根据需要调整休眠时间

        logging.error(f"超时：等待文件 {remote_path} 超过 {timeout} 秒。")
        return False

    def close(self):
        self.client.close()



# -*- coding: utf-8 -*-            
# @Author : buyfakett
# @Time : 2023/12/15 10:17
import logging
from typing import Union, List


def edit_uuid(uuid_filename: Union[str, List[str]]):
    """
    把uuid的第一个值提取出来，做一层文件夹，可以让oss上快速查询
    :param uuid_filename: uuid filename
    :return: 解析后的filename
    """
    if isinstance(uuid_filename, str):
        first_part = uuid_filename.split('-')[0]
        uuid_url = first_part + '/' + uuid_filename
        return uuid_url
    elif isinstance(uuid_filename, List):
        uuid_url = []
        for i in range(len(uuid_filename)):
            first_part = uuid_filename[i].split('-')[0]
            uuid_url.append(first_part + '/' + uuid_filename[i])
        return uuid_url
    else:
        logging.error('入参类型不对')
        return ''


def edit_url(uuid_url: Union[str, List[str]]):
    """
    把第一层文件夹给去掉
    :param uuid_url: uuid url
    :return: 解析后的filename
    """
    if isinstance(uuid_url, str):
        file_name = uuid_url.split('/')[-1]
        return file_name
    elif isinstance(uuid_url, List):
        for i in range(len(uuid_url)):
            uuid_url[i] = uuid_url[i].split('/')[-1]
        return uuid_url
    else:
        logging.error('入参类型不对')
        return ''
